*DECK AXGSYM
      SUBROUTINE AXGSYM( IPGEOM,   IPRT, NBLOCK,  NTYPO,   NXYZ,
     >                   GEONAM, LCLSYM,
     >                   MINGRI, MAXGRI,  CELLT,
     >                   KEYTYP, ITGEOM)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Unfold assembly or cell according to center cell symmetry in
* $x$, $y$ or $z$ and verify if the symmetry is valid.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Roy and G. Marleau
*
*Parameters: input
* IPGEOM  pointer to the reference geometry data structure.
* IPRT    intermediate printing level for output.
* NBLOCK  number of block in geometry.
* NTYPO   number of types in geometry.
* NXYZ    maximum mesh size in directions $x$, $y$ and $z$.
* GEONAM  name of the reference geometry.
* LCLSYM  flag that is set to 1 when the $x$ (\verb|LCLSYM(1)|),
*         $y$ (\verb|LCLSYM(2)|) and/or $z$ (\verb|LCLSYM(3)|) 
*         symmetries are required. 
* MINGRI  minimum grid cell in $x$, $y$ and $z$ directions.
* MAXGRI  maximum grid cell in $x$, $y$ and $z$ directions.
* CELLT   cell type name.
*
*Parameters: input/output
* KEYTYP  type key for each block.
* ITGEOM  turn key associated with each cell type.
*
*External functions
* LELCSY  to verify if a geometry possesses the required internal
*         symmetry.
* AXGTRS  to modify current turn according to required internal
*         symmetry.
* AXGTRN  to associate a DRAGON turn name to a specific turn key. 
*
*-----------------------------------------------------------------------
*
      IMPLICIT           NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER            IPGEOM,IPRT,NBLOCK,NTYPO,NXYZ
      CHARACTER          GEONAM*12
      INTEGER            LCLSYM(3)
      INTEGER            MINGRI(3),MAXGRI(3),CELLT(3*NTYPO)
      INTEGER            KEYTYP(NBLOCK),ITGEOM(NBLOCK)
*----
*  EXTERNAL FUNCTIONS
*----
      LOGICAL            LELCSY
      INTEGER            AXGTRS
      CHARACTER          AXGTRN*2
*----
*  LOCAL VARIABLES
*----
      INTEGER            IOUT
      CHARACTER          NAMSBR*6
      PARAMETER         (IOUT=6,NAMSBR='AXGSYM')
C----
C  MEMORY ALLOCATION
C----
      INTEGER            IBASE(1)
      REAL               RBASE
      COMMON             RBASE(1)
      EQUIVALENCE       (RBASE(1),IBASE(1))
      INTEGER            IMESH,ISPLT,IMESH1,ISPLT1
*----
*  LOCAL PARAMETERS
*----
      INTEGER            IX,IY,IZ,IOF1,IOF2
      INTEGER            IKG,IKT(2)
      LOGICAL            VALSYM
      CHARACTER          GEOCV*12 
*----
*  APPLY SYMMETRY IN Z
*----
      CALL SETARA(RBASE,(NXYZ+1),IMESH)
      CALL SETARA(IBASE,NXYZ,ISPLT)
      CALL SETARA(RBASE,2*3*3*(NXYZ+1),IMESH1)
      CALL SETARA(IBASE,3*3*NXYZ,ISPLT1)
      IF( LCLSYM(3) .NE. 0) THEN
        DO 200 IZ=1,MINGRI(3)
          DO 210 IY=1,MAXGRI(2)
            DO 220 IX=1,MAXGRI(1)
              IOF1=((IZ-1)*MAXGRI(2)+(IY-1))*MAXGRI(1)+IX
              IOF2=((MAXGRI(3)-IZ)*MAXGRI(2)+(IY-1))*MAXGRI(1)+IX
              IF( KEYTYP(IOF1) .NE. KEYTYP(IOF2)) THEN
                IF( KEYTYP(IOF1) .GT. KEYTYP(IOF2)) THEN
                  KEYTYP(IOF2)= KEYTYP(IOF1)
                  ITGEOM(IOF2)= AXGTRS(ITGEOM(IOF1),4)
                ELSE
                  KEYTYP(IOF1)= KEYTYP(IOF2)
                  ITGEOM(IOF1)= AXGTRS(ITGEOM(IOF2),4)
                ENDIF
              ENDIF
              IF(IZ .EQ. (MAXGRI(3)+1-IZ)) THEN
                IKG=KEYTYP(IOF1)
                IKT(1)=ITGEOM(IOF1)
                IKT(2)=AXGTRS(IKT(1),4)
                IF(IKG .GT. 0) THEN 
                  WRITE(GEOCV,'(3A4)')
     >            CELLT(3*IKG-2),CELLT(3*IKG-1),CELLT(3*IKG)
                  IF(GEOCV .EQ. '            ') THEN       
                    IF(IPRT .GT. 10) WRITE(IOUT,8000) NAMSBR,'Z-Z',
     >              GEONAM,AXGTRN(IKT(1)),AXGTRN(IKT(2))
                  ELSE
                    IF(IPRT .GT. 10) WRITE(IOUT,8000) NAMSBR,'Z-Z',
     >              GEOCV,AXGTRN(IKT(1)),AXGTRN(IKT(2))
                  ENDIF
                  VALSYM=LELCSY(IPGEOM,IPRT,GEONAM,GEOCV,NXYZ,IKT,
     >                          RBASE(IMESH) ,IBASE(ISPLT),
     >                          RBASE(IMESH1),IBASE(ISPLT1))
                  IF(.NOT. VALSYM) THEN 
                    WRITE(IOUT,8001) 'Z-Z',GEOCV,
     >              AXGTRN(IKT(1)),AXGTRN(IKT(2))
                    CALL XABORT(NAMSBR//': INVALID Z SYMMETRY FOR CELL')
                  ENDIF
                ENDIF
              ENDIF
 220        CONTINUE
 210      CONTINUE
 200    CONTINUE
      ENDIF
*----
*  APPLY SYMMETRY IN Y
*----
      IF( LCLSYM(2).NE.0)THEN
        DO 300 IZ=1,MAXGRI(3)
          DO 310 IY=1,MINGRI(2)
            DO 320 IX=1,MAXGRI(1)
              IOF1=((IZ-1)*MAXGRI(2)+(IY-1))*MAXGRI(1)+IX
              IOF2=((IZ-1)*MAXGRI(2)+(MAXGRI(2)-IY))*MAXGRI(1)+IX
              IF(KEYTYP(IOF1) .NE. KEYTYP(IOF2)) THEN
                IF(KEYTYP(IOF1) .GT. KEYTYP(IOF2)) THEN
                  KEYTYP(IOF2)= KEYTYP(IOF1)
                  ITGEOM(IOF2)= AXGTRS(ITGEOM(IOF1),2)
                ELSE
                  KEYTYP(IOF1)= KEYTYP(IOF2)
                  ITGEOM(IOF1)= AXGTRS(ITGEOM(IOF2),2)
                ENDIF
              ENDIF
              IF(IY .EQ. (MAXGRI(2)+1-IY) ) THEN
                IKG=KEYTYP(IOF1)
                IKT(1)=ITGEOM(IOF1)
                IKT(2)=AXGTRS(IKT(1),2)
                IF(IKG .GT. 0) THEN 
                  WRITE(GEOCV,'(3A4)')
     >            CELLT(3*IKG-2),CELLT(3*IKG-1),CELLT(3*IKG)       
                  IF(GEOCV .EQ. '            ') THEN       
                    IF(IPRT .GT. 10)  WRITE(IOUT,8000) NAMSBR,'Y-Y',
     >                GEONAM,AXGTRN(IKT(1)),AXGTRN(IKT(2))
                  ELSE
                    IF(IPRT .GT. 10)  WRITE(IOUT,8000) NAMSBR,'Y-Y',
     >                GEOCV,AXGTRN(IKT(1)),AXGTRN(IKT(2))
                  ENDIF
                  VALSYM=LELCSY(IPGEOM,IPRT,GEONAM,GEOCV,NXYZ,IKT,
     >                          RBASE(IMESH) ,IBASE(ISPLT),
     >                          RBASE(IMESH1),IBASE(ISPLT1))
                  IF(.NOT. VALSYM) THEN 
                    WRITE(IOUT,8001) 'Y-Y',GEOCV,
     >              AXGTRN(IKT(1)),AXGTRN(IKT(2))
                    CALL XABORT(NAMSBR//': INVALID Y SYMMETRY FOR CELL')
                  ENDIF
                ENDIF
              ENDIF
 320        CONTINUE
 310      CONTINUE
 300    CONTINUE
      ENDIF
*----
*  APPLY SYMMETRY IN X
*----
      IF( LCLSYM(1).NE.0)THEN
        DO 400 IZ=1,MAXGRI(3)
          DO 410 IY=1,MAXGRI(2)
            DO 420 IX=1,MINGRI(1)
              IOF1=((IZ-1)*MAXGRI(2)+(IY-1))*MAXGRI(1)+IX
              IOF2=((IZ-1)*MAXGRI(2)+(IY-1))*MAXGRI(1)+MAXGRI(1)+1-IX
              IF(KEYTYP(IOF1) .NE. KEYTYP(IOF2)) THEN
                IF(KEYTYP(IOF1) .GT. KEYTYP(IOF2)) THEN
                  KEYTYP(IOF2)= KEYTYP(IOF1)
                  ITGEOM(IOF2)= AXGTRS(ITGEOM(IOF1),1)
                ELSE
                  KEYTYP(IOF1)= KEYTYP(IOF2)
                  ITGEOM(IOF1)= AXGTRS(ITGEOM(IOF2),1)
                ENDIF
              ENDIF
              IF(IX .EQ. (MAXGRI(1)+1-IX)) THEN
                IKG=KEYTYP(IOF1)
                IKT(1)=ITGEOM(IOF1)
                IKT(2)=AXGTRS(IKT(1),1)
                IF(IKG .GT. 0) THEN 
                  WRITE(GEOCV,'(3A4)')
     >            CELLT(3*IKG-2),CELLT(3*IKG-1),CELLT(3*IKG)       
                  IF(GEOCV .EQ. '            ') THEN
                    IF(IPRT .GT. 10) WRITE(IOUT,8000) NAMSBR,'X-X',       
     >              GEONAM,AXGTRN(IKT(1)),AXGTRN(IKT(2))
                  ELSE
                    IF(IPRT .GT. 10) WRITE(IOUT,8000) NAMSBR,'X-X',       
     >              GEOCV,AXGTRN(IKT(1)),AXGTRN(IKT(2))
                  ENDIF
                  VALSYM=LELCSY(IPGEOM,IPRT,GEONAM,GEOCV,NXYZ,IKT,
     >                          RBASE(IMESH) ,IBASE(ISPLT),
     >                          RBASE(IMESH1),IBASE(ISPLT1))
                  IF(.NOT. VALSYM) THEN 
                    WRITE(IOUT,8001) 'X-X',GEOCV,
     >              AXGTRN(IKT(1)),AXGTRN(IKT(2))
                    CALL XABORT(NAMSBR//': INVALID X SYMMETRY FOR CELL')
                  ENDIF
                ENDIF
              ENDIF
 420        CONTINUE
 410      CONTINUE
 400    CONTINUE
      ENDIF
      CALL RLSARA(IBASE(ISPLT1))
      CALL RLSARA(RBASE(IMESH1))
      CALL RLSARA(IBASE(ISPLT))
      CALL RLSARA(RBASE(IMESH))
*----
*  RETURN
*----
      RETURN 
*----
*  FORMAT
*----
 8000 FORMAT(1X,A6,' NOW TESTING SYMMETRY ',A3,' FOR ',
     > A12,1X,'WITH ROTATION',1X,A2,' AND ',A2)
 8001 FORMAT('            INVALID SYMMETRY ',A3,' FOR ',
     > A12,1X,'WITH ROTATION',1X,A2,' AND ',A2)
      END
