*DECK CPOMAR
      SUBROUTINE CPOMAR(IPEDIT,NGROUP,NMERGE,NL    ,NIFISS,NEDMAC,
     >                  HVECT ,IVECT ,NPROC ,ILEAKS,DXSMAC,DSCMAC,
     >                  EMJMAC,DISFC ,SCATC ,IJJ   ,NJJ   ,DNUFI )
C
C---------------------------  CPOMAR  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : CPOMAR
C      USE      : GET MACROSCOPIC CROSS SECTION FROM IPEDIT
C      MODIFIED : 97-07-18 (G.M.)
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C    INPUT
C      IPEDIT   : POINTER TO THE EDIT                I
C      NGROUP   : NUMBER OF GROUPS CONDENSED         I
C      NMERGE   : NUMBER OF REGIONS MERGED           I
C      NL       : NUMBER OF LEGENDRE ORDERS          I
C      NIFISS   : NUMBER OF FISSILE ISOTOPES         I
C      NEDMAC   : NUMBER OF EXTRA EDIT VECTORS       I
C      HVECT    : NAME OF ADD XS.                    C(NEDMAC)*6
C      IVECT    : LOCATOIN OF ADD XS.                I(NEDMAC)
C      NPROC    : NUMBER OF MIC XS TO PROCESS        I
C      ILEAKS   : LEAK CALCULATION                   I
C                 = 0 NO LEAKAGE
C                 = 1 LEAKAGE BY SURFACE (ALBEDO>0.0)
C                 = 2 HOMOGENEOUS LEAKAGE COEFFICIENTS
C                 = 3 DIRECTIONAL LEAKAGE COEFFICIENTS
C    OUTPUT
C      DXSMAC   : AVERAGED REGION/GROUP X-S          D(NGROUP,NPROC,
C                                                          NMERGE)
C      DSCMAC   : DOUBLE PRECISION SCATTERING RATES  D(NGROUP,NGROUP,
C                                                       NL,NMERGE)
C      DISFC    : DISADVANTAGE FACTOR                R(NGROUP)
C      EMJMAC   : ENERGY PER FISSION                 R(NMERGE)
C    WORK
C      SCATC    : COMPRESS SCATTERING                R(NMERGE*
C                                             MAX(NGROUP,NIFISS))
C      IJJ      : POSITION OF FIRST DIFFUSION GROUP  I(NMERGE)
C      NJJ      : NUMBER OF DIFFUSION GROUP          I(NMERGE)
C      DNUFI    : FISSION SOURCE                     D(NMERGE,
C                                                      NIFISS+1)
C
C
C---------------------------   CPOMAR  --------------------------------
C
      IMPLICIT         NONE
      INTEGER          IPEDIT,NGROUP,NMERGE,NL,NIFISS,NEDMAC,
     >                 IVECT(NEDMAC),NPROC,ILEAKS,IJJ(NMERGE),
     >                 NJJ(NMERGE)
      CHARACTER        HVECT(NEDMAC)*6
      REAL             DISFC(NGROUP),SCATC(NMERGE*NGROUP),
     >                 EMJMAC(NMERGE)
      DOUBLE PRECISION DXSMAC(NGROUP,NPROC,NMERGE),
     >                 DSCMAC(NGROUP,NGROUP,NL,NMERGE),
     >                 DNUFI(NMERGE,NIFISS+1)
C----
C  LOCAL PARAMETERS
C----
      INTEGER          NDPROC,ILCMUP,ILCMDN
      PARAMETER       (NDPROC=20,ILCMUP=1,ILCMDN=2)
      INTEGER          IGR,IED,IXSR,JXSR,KXSR,IMRG,IFIS,ILOCED,
     >                 IL,IPOSIT,JGR1,JGR2,JGR,ILCMLN,ITYLCM
      CHARACTER        CM*2,CGRPNM*12
      DOUBLE PRECISION DEPRO
C----
C  INITIALIZE REACTION RATE VECTOR
C----
      CALL XDRDBL(DXSMAC,NGROUP*NPROC*NMERGE,0.0D0)
      CALL XDRDBL(DSCMAC,NGROUP*NGROUP*NL*NMERGE,0.0D0)
      CALL XDRDBL(DNUFI,NMERGE*(NIFISS+1),0.0D0)
C----
C  READ ALL CROSS SECTION FROM IPEDIT EXCEPT CHI
C----
      CALL LCMLEN(IPEDIT,'FLUXDISAFACT',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.NGROUP) THEN
        CALL LCMGET(IPEDIT,'FLUXDISAFACT',DISFC)
      ELSE
        CALL XDRSET(DISFC,NGROUP,0.0)
      ENDIF
      CALL LCMLEN(IPEDIT,'MACRO-EMJ',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.NMERGE) THEN
        CALL LCMGET(IPEDIT,'MACRO-EMJ',EMJMAC)
      ELSE
        CALL XDRSET(EMJMAC,NMERGE,0.0)
      ENDIF
      DO 100 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPEDIT,CGRPNM,ILCMUP)
        IF(NEDMAC.GT.0) THEN
          DO 110 IED=1,NEDMAC
            IXSR=IVECT(IED)
            IF(IXSR.GT.0) THEN
              CALL LCMGET(IPEDIT,HVECT(IED),SCATC)
              DO 111 IMRG=1,NMERGE
                DXSMAC(IGR,IXSR,IMRG)=DBLE(SCATC(IMRG))
 111          CONTINUE
            ENDIF
 110      CONTINUE
        ENDIF
        IXSR=13
        CALL LCMLEN(IPEDIT,'OVERV',ILCMLN,ITYLCM)
        IF(ILCMLN.EQ.NMERGE) THEN
          CALL LCMGET(IPEDIT,'OVERV',SCATC)
          DO 120 IMRG=1,NMERGE
            DXSMAC(IGR,IXSR,IMRG)=DBLE(SCATC(IMRG))
 120      CONTINUE
        ENDIF
        IXSR=1
        CALL LCMGET(IPEDIT,'TOTAL',SCATC)
        DO 130 IMRG=1,NMERGE
          DXSMAC(IGR,IXSR,IMRG)=DBLE(SCATC(IMRG))
 130    CONTINUE
        IXSR=2
        CALL LCMGET(IPEDIT,'TRANC',SCATC)
        DO 170 IMRG=1,NMERGE
          DXSMAC(IGR,IXSR,IMRG)=DBLE(SCATC(IMRG))
 170    CONTINUE
        IXSR=16
        CALL LCMGET(IPEDIT,'FLUX-INTG',SCATC)
        DO 190 IMRG=1,NMERGE
          DXSMAC(IGR,IXSR,IMRG)=DBLE(SCATC(IMRG))
 190    CONTINUE
        IF(NIFISS.GT.0) THEN
          IXSR=3
          JXSR=16
          CALL LCMGET(IPEDIT,'NUSIGF',SCATC)
          ILOCED=1
          DO 150 IFIS=1,NIFISS
            DO 151 IMRG=1,NMERGE
              DXSMAC(IGR,IXSR,IMRG)=DXSMAC(IGR,IXSR,IMRG)
     >          +DBLE(SCATC(ILOCED))
              DNUFI(IMRG,IFIS)=DNUFI(IMRG,IFIS)
     >          +DBLE(SCATC(ILOCED))*DXSMAC(IGR,JXSR,IMRG)
              ILOCED=ILOCED+1
 151        CONTINUE
 150      CONTINUE
          IXSR=4
          CALL LCMGET(IPEDIT,'NFTOT',SCATC)
          ILOCED=1
          DO 152 IFIS=1,NIFISS
            DO 153 IMRG=1,NMERGE
              DXSMAC(IGR,IXSR,IMRG)=DXSMAC(IGR,IXSR,IMRG)
     >          +DBLE(SCATC(ILOCED))
              ILOCED=ILOCED+1
 153        CONTINUE
 152      CONTINUE
        ENDIF
        IXSR=NDPROC
        DO 200 IL=1,NL
          IXSR=IXSR+1
          WRITE (CM,'(I2)') IL-1
          CALL LCMGET(IPEDIT,'SCAD'//CM,SCATC)
          CALL LCMGET(IPEDIT,'NJJD'//CM,NJJ)
          CALL LCMGET(IPEDIT,'IJJD'//CM,IJJ)
          IPOSIT=0
          DO 210 IMRG=1,NMERGE
            JGR2=IJJ(IMRG)
            JGR1=JGR2-NJJ(IMRG)+1
            DO 211 JGR=JGR2,JGR1,-1
              IPOSIT=IPOSIT+1
              DSCMAC(IGR,JGR,IL,IMRG)=DBLE(SCATC(IPOSIT))
              DXSMAC(JGR,IXSR,IMRG)=DXSMAC(JGR,IXSR,IMRG)
     >                             +DSCMAC(IGR,JGR,IL,IMRG)
 211        CONTINUE
 210      CONTINUE
 200    CONTINUE
        IF(ILEAKS.EQ.2) THEN
          IXSR=17
          CALL LCMGET(IPEDIT,'DIFFHOM',SCATC)
          DO 180 IMRG=1,NMERGE
            IF(SCATC(IMRG).GT.0.0) THEN
              DXSMAC(IGR,IXSR,IMRG)=1.0D0/(3.0D0*DBLE(SCATC(IMRG)))
            ENDIF
 180      CONTINUE
        ELSE IF(ILEAKS.EQ.3) THEN
          IXSR=17
          CALL LCMGET(IPEDIT,'DIFFHOM',SCATC)
          DO 181 IMRG=1,NMERGE
            IF(SCATC(IMRG).GT.0.0) THEN
              DXSMAC(IGR,IXSR,IMRG)=1.0D0/(3.0D0*DBLE(SCATC(IMRG)))
            ENDIF
 181      CONTINUE
          IXSR=18
          CALL LCMGET(IPEDIT,'DIFFX',SCATC)
          DO 182 IMRG=1,NMERGE
            IF(SCATC(IMRG).GT.0.0) THEN
              DXSMAC(IGR,IXSR,IMRG)=1.0D0/(3.0D0*DBLE(SCATC(IMRG)))
            ENDIF
 182      CONTINUE
          IXSR=19
          CALL LCMGET(IPEDIT,'DIFFY',SCATC)
          DO 183 IMRG=1,NMERGE
            IF(SCATC(IMRG).GT.0.0) THEN
              DXSMAC(IGR,IXSR,IMRG)=1.0D0/(3.0D0*DBLE(SCATC(IMRG)))
            ENDIF
 183      CONTINUE
          IXSR=20
          CALL LCMGET(IPEDIT,'DIFFZ',SCATC)
          DO 184 IMRG=1,NMERGE
            IF(SCATC(IMRG).GT.0.0) THEN
              DXSMAC(IGR,IXSR,IMRG)=1.0D0/(3.0D0*DBLE(SCATC(IMRG)))
            ENDIF
 184      CONTINUE
        ELSE
          IXSR=17
          JXSR=1
          IF(NL.GE.2) THEN
            KXSR=NDPROC+2
            DO 185 IMRG=1,NMERGE
              DXSMAC(IGR,IXSR,IMRG)=DXSMAC(IGR,JXSR,IMRG)
     >                             -DXSMAC(IGR,KXSR,IMRG)
 185        CONTINUE
          ELSE
            DO 186 IMRG=1,NMERGE
              DXSMAC(IGR,IXSR,IMRG)=DXSMAC(IGR,JXSR,IMRG)
 186        CONTINUE
          ENDIF
        ENDIF
        CALL LCMSIX(IPEDIT,CGRPNM,ILCMDN)
 100  CONTINUE
C----
C  PROCESS CHI IF REQUIRED
C----
      IF(NIFISS.GT.0) THEN
        DO 160 IGR=1,NGROUP
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
          CALL LCMSIX(IPEDIT,CGRPNM,ILCMUP)
          IXSR=5
          CALL LCMGET(IPEDIT,'CHI',SCATC)
          ILOCED=1
          DO 161 IFIS=1,NIFISS
            DO 162 IMRG=1,NMERGE
              DXSMAC(IGR,IXSR,IMRG)=DXSMAC(IGR,IXSR,IMRG)
     >          +DBLE(SCATC(ILOCED))*DNUFI(IMRG,IFIS)
              ILOCED=ILOCED+1
 162        CONTINUE
 161      CONTINUE
          CALL LCMSIX(IPEDIT,CGRPNM,ILCMDN)
 160    CONTINUE
      ENDIF
C----
C  FIND TOTAL ENERGY PRODUCTION
C----
      IXSR=4
      JXSR=16
      DO 250 IMRG=1,NMERGE
        DEPRO=0.0D0
        DO 251 IGR=1,NGROUP
          DEPRO=DEPRO+DXSMAC(IGR,IXSR,IMRG)*DXSMAC(IGR,JXSR,IMRG)
     >         *DBLE(EMJMAC(IMRG))
 251    CONTINUE
        EMJMAC(IMRG)=REAL(DEPRO)
 250  CONTINUE
      RETURN
      END
