*DECK LIBDEN
      SUBROUTINE LIBDEN(IPLIB,NGROUP,NBISO,NBMIX,NL,NED,NNFI,MXXS,
     >                  ISONAM,ISONFI,MIX,DEN,MASKK,MASK,MASKG,HVECT,
     >                  IVECT,ITRANC,IFGUPS,IPXS,IPSC,NIFIS,NOFIS,
     >                  GAS,GXS,SCT,NJJ,IJJ,IPOS,SCAT,ISTATE,
     >                  ITSTMP,TMPDAY,SCTISO,NPROC,INDPRO,ITYPRO,XSREC)
C
C-----------------------------------------------------------------------
C
C TRANSFORMATION OF THE ISOTOPE ORDERED MICROSCOPIC CROSS SECTIONS TO
C GROUP ORDERED MACROSCOPIC CROSS SECTIONS (PART 2).
C
C INPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C            (L_LIBRARY SIGNATURE).
C  NGROUP  : NUMBER OF ENERGY GROUPS.
C  NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  NBMIX   : NUMBER OF MIXTURES PRESENT IN THE CALCULATION DOMAIN.
C  NL      : MAXIMUM NUMBER OF LEGENDRE ORDERS REQUIRED
C  NED     : MAXIMUM NUMBER OF EDIT XS
C  NNFI    : MAXIMUM NUMBER OF FISSILE ISOTOPES
C  MXXS    : NUMBER OF VECTOR XS
C  ISONAM  : NAME OF EACH ISOTOPE. ISONAM(3,NBISO)
C  ISONFI  : NAMES OF FISSILE ISOTOPES. ISONFI(2,NBISO)
C  MIX     : MIXTURE NUMBER OF EACH ISOTOPE. CAN BE ZERO.
C            DIMENSION MIX(NBISO)
C  DEN     : DENSITY OF EACH ISOTOPE. DIMENSION DEN(NBISO)
C  MASKK   : =.TRUE. IF A MACROLIB EXISTS
C  MASK    : =.TRUE. IF A MIXTURE IS TO BE MADE. DIMENSION MASK(NBMIX)
C  MASKG   : =.TRUE. IF AN ENERGY GROUP IS TO BE TREATED.
C            DIMENSION MASKG(NGROUP)
C  HVECT   : CHARACTER NAMES OF EDITS XS, DIMENSION HVECT(NED)
C  IVECT   : POINTER FOR EDITS XS. DIMENSION IVECT(NED)
C  ITRANC  : TYPE OF TRANSPORT CORRECTIONS. =0: NO TRANSPORT CORRECTION;
C            =-1: APOLLO TYPE TRANSPORT CORRECTION; >0: WIMS TYPE
C            TRANSPORT CORRECTION WHERE ITRANC IS THE GROUP LIMIT.
C  ISTATE  : INITIAL INFORMATION ON MACROLIB.
C  ITSTMP  : = 0 PERTURBATION FORBIDDEN
C            = 1 PERTURBATION NOT USED EVEN IF PRESENT
C            = 2 PERTURBATION USED IF PRESENT
C  TMPDAY  : TIME IN DAYS/BURNUP/IRRADIATION STAMP TMPDAY(3)
C  NPROC   : MAXIMUM NUMBER OF XS TO PROCESS
C  INDPRO  : FLAG FOR XS TO PROCESS INDPRO(NPROC)
C
C SCRATCH STORAGE:
C  IFGUPS  : DIMENSION IFGUPS(NBMIX,NL)
C  IPXS    : DIMENSION IPXS(MXXS)
C  IPSC    : DIMENSION IPSC(MXNL)
C  NIFIS   : DIMENSION NIFIS(NBMIX,NNFI)
C  NOFIS   : DIMENSION NOFIS(NBISO)
C  GAS     : DIMENSION GAS(MAX(NGROUP*NGROUP,NBMIX**2,3*NBMIX*NNFI))
C  GXS     : DIMENSION GXS(NBMIX,MXXS,NGROUP)
C  SCT     : DIMENSION SCT(NBMIX,NGROUP,NL,NGROUP)
C  NJJ     : DIMENSION NJJ(MAX(NGROUP,NBMIX))
C  IJJ     : DIMENSION IJJ(MAX(NGROUP,NBMIX))
C  IPOS    : DIMENSION IPOS(MAX(NGROUP,NBMIX))
C  SCAT    : DIMENSION SCAT(NGROUP*NBMIX)
C  SCTISO  : DIMENSION SCTISO(NGROUP,NGROUP,NL)
C  ITYPRO  : DIMENSION INDPRO(NPROC)
C  XSREC   : DIMENSION XSREC(NGROUP,NPROC)
C
C                                        AUTHOR: A. HEBERT 19/04/88
C--------------------------------------- MODIFY: G. M.     95/12/01 ----
C
      IMPLICIT NONE
C-----
C PARAMETERS
C-----
      INTEGER       IOUT,NSTATE,NDPROC
      PARAMETER    (IOUT=6,NSTATE=20,NDPROC=20)
C----
C MEMORY ALLOCATION
C----
      REAL          RBASE(1)
      INTEGER       IBASE(1)
      COMMON        RBASE
      EQUIVALENCE  (RBASE(1),IBASE(1))
      INTEGER       IADDXS
C-----
C DIMENSIONS
C-----
      INTEGER       NBMIX,NL,MXXS,NBISO,NED,NPROC,NGROUP
C----
C LOCAL VARIABLES
C----
      INTEGER       IPLIB,NXSPER,LENGTH,ITYLCM,ISO,IMX,ITC,IXSPER,
     1              IL,IXS,IG1,IG2,IED,JNFL,NNFI,JCHL,ISOTF,
     2              JFTL,IXSNF,IXSFT,IXSCH,NEDU,JL,I,NOMIX,IPOSDE,
     3              IGFIN,IGDEB,IGMIN,IGMAX,ILCNF,ILCFT,ILCCH,IFIS,
     4              ITRANC
C
      INTEGER       MIX(*),IPXS(*),IPSC(*),NIFIS(NBMIX,*),NOFIS(*),
     1              NJJ(*),IJJ(*),IPOS(*),ISTATE(NSTATE),
     2              IFGUPS(NBMIX,NL),KCHAR(3),ITSTMP,ISONAM(3,NBISO),
     3              ISONFI(2,NBISO),IVECT(NED),INDPRO(NPROC),
     4              ITYPRO(NPROC)
C
      REAL          TIMFCT,XTF,DENISO
      REAL          DEN(NBISO),GAS(*),GXS(NBMIX,MXXS,NGROUP),
     &              SCT(NBMIX,NGROUP,NL,NGROUP),SCAT(NGROUP*NBMIX),
     &              SCTISO(NGROUP,NGROUP,NL),XSREC(NGROUP,NPROC),
     &              TMPDAY(3),TMPPER(2,3)
      CHARACTER     CGRPNM*12,CM*2,TEXT12*12,HNAMIS*12,
     1              HVECT(*)*(*)
      LOGICAL       MASKK,MASK(*),MASKG(*),LOGL
C----
C  INITIALIZE STORAGE VECTOR
C----
      CALL XDRSET(SCT,NGROUP*NGROUP*NBMIX*NL,0.0)
      CALL XDRSET(GXS,NGROUP*NBMIX*MXXS,0.0)
      NXSPER=1
      CALL LCMLEN(IPLIB,'TIMESPER',LENGTH,ITYLCM)
      IF(LENGTH.GE.2.AND.LENGTH.LE.6) THEN
        IF(ITSTMP.EQ.0) THEN
          CALL XABORT('LIBDEN: XS PERTURBATION FORBIDDEN')
        ELSE IF(ITSTMP.EQ.2) THEN
          CALL LCMGET(IPLIB,'TIMESPER',TMPPER)
          TIMFCT=TMPDAY(1)-TMPPER(1,1)
          XTF=TIMFCT/TMPPER(2,1)
          IF(XTF.NE.0.0) THEN
             NXSPER=2
          ENDIF
          IF(XTF.LT.0.0) THEN
            WRITE(IOUT,6000) TMPPER(1,1),TMPDAY(1)
          ELSE IF(XTF.GT.1.0) THEN
            WRITE(IOUT,6001) TMPPER(1,1)+TMPPER(2,1),TMPDAY(1)
          ENDIF
        ENDIF
      ENDIF
C----
C  PROCESS ISOTOPES IN MIXTURE TO BE UPDATED
C----
      DO 1000 ISO=1,NBISO
        IMX=MIX(ISO)
        DENISO=DEN(ISO)
        ISOTF=NOFIS(ISO)
        IF(IMX.EQ.0) GO TO 1015
        IF((.NOT.MASK(IMX)).AND.MASKK) GO TO 1015
      IF(DENISO.NE.0.0) THEN
        WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,ISO),ITC=1,3)
        CALL LCMSIX(IPLIB,HNAMIS,1)
        DO 1010 IXSPER=1,NXSPER
          CALL XDRLGS(IPLIB,-1,NPROC,0,NL-1,IXSPER,NGROUP,INDPRO,
     >                ITYPRO,XSREC,SCTISO,GAS,NJJ,IJJ)
C----
C  SCATTERING
C----
          DO 1020 IL=1,NL
            IXS=NDPROC+IL
            IF(ITYPRO(IXS).GT.0) THEN
              IPSC(IL)=1
              DO 1021 IG1=1,NGROUP
                DO 1022 IG2=1,NGROUP
                  SCT(IMX,IG2,IL,IG1)=SCT(IMX,IG2,IL,IG1)
     >                   +SCTISO(IG1,IG2,IL)*DENISO
 1022           CONTINUE
 1021         CONTINUE
            ENDIF
 1020     CONTINUE
C----
C  TOTAL AND TRANSPORT XS
C----
          IL=1
          IXS=1
          IF(ITYPRO(IXS).GT.0) THEN
            IPXS(IL)=1
            DO 1030 IG1=1,NGROUP
              GXS(IMX,IL,IG1)=GXS(IMX,IL,IG1)+XSREC(IG1,IXS)*DENISO
 1030       CONTINUE
          ENDIF
          IL=IL+1
          IXS=2
          IF(ITYPRO(IXS).GT.0) THEN
            IPXS(IL)=1
            DO 1040 IG1=1,NGROUP
              GXS(IMX,IL,IG1)=GXS(IMX,IL,IG1)+XSREC(IG1,IXS)*DENISO
 1040       CONTINUE
          ENDIF
C----
C  EDITING
C----
          DO 1050 IED=1,NED
            IL=IL+1
            IXS=IVECT(IED)
            IF(IXS.GT.0) THEN
              IF(ITYPRO(IXS).GT.0) THEN
                IPXS(IL)=1
                DO 1051 IG1=1,NGROUP
                  GXS(IMX,IL,IG1)=GXS(IMX,IL,IG1)+XSREC(IG1,IXS)*DENISO
 1051           CONTINUE
              ENDIF
            ENDIF
 1050     CONTINUE
C----
C  PROCESS FISSION INFORMATION
C----
          IL=2+NED
          IF(ISOTF.NE.0) THEN
            JNFL=IL+ISOTF
            JFTL=JNFL+NNFI
            JCHL=JFTL+NNFI
            IXSNF=3
            IXSFT=4
            IXSCH=5
            IF(ITYPRO(IXSNF).GT.0) THEN
              IPXS(JNFL)=1
              IPXS(JFTL)=1
              IPXS(JCHL)=1
              DO  1060 IG1=1,NGROUP
                GXS(IMX,JNFL,IG1)=GXS(IMX,JNFL,IG1)
     >                           +XSREC(IG1,IXSNF)*DENISO
                GXS(IMX,JFTL,IG1)=GXS(IMX,JFTL,IG1)
     >                           +XSREC(IG1,IXSFT)*DENISO
                GXS(IMX,JCHL,IG1)=GXS(IMX,JCHL,IG1)
     >                           +XSREC(IG1,IXSCH)
 1060         CONTINUE
            ENDIF
          ENDIF
          DENISO=DENISO*TIMFCT
 1010   CONTINUE
        CALL LCMSIX(IPLIB,' ',2)
      ELSE
        WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,ISO),ITC=1,3)
        CALL LCMSIX(IPLIB,HNAMIS,1)
        DO 1011 IXSPER=1,NXSPER
          CALL XDRLGS(IPLIB,-1,NPROC,0,NL-1,IXSPER,NGROUP,INDPRO,
     >                ITYPRO,XSREC,SCTISO,GAS,NJJ,IJJ)
C----
C  PROCESS FISSION INFORMATION
C----
          IL=2+NED
          IF(ISOTF.NE.0) THEN
            JNFL=IL+ISOTF
            JFTL=JNFL+NNFI
            JCHL=JFTL+NNFI
            IXSNF=3
            IXSFT=4
            IXSCH=5
            IF(ITYPRO(IXSNF).GT.0) THEN
              IPXS(JNFL)=1
              IPXS(JFTL)=1
              IPXS(JCHL)=1
              DO  1061 IG1=1,NGROUP
                GXS(IMX,JCHL,IG1)=GXS(IMX,JCHL,IG1)
     >                           +XSREC(IG1,IXSCH)
 1061         CONTINUE
            ENDIF
          ENDIF
 1011   CONTINUE
        CALL LCMSIX(IPLIB,' ',2)
      ENDIF
 1015   CONTINUE
 1000 CONTINUE
C----
C  CHECK IF AT LEAST ONE EDIT AND ONE FISSILE ISOTOPE AND
C  STORE NAME
C----
      CALL LCMSIX(IPLIB,'MACROLIB',1)
      CALL SETARA(IBASE,2*MAX(NED,1),IADDXS)
      NEDU=0
      JL=2
      DO 1070 IED=1,NED
        JL=JL+1
        IF(IPXS(JL).NE.0) THEN
          NEDU=NEDU+1
          READ(HVECT(IED),'(A4,A2)')
     >       (IBASE(IADDXS+2*(NEDU-1)+I),I=0,1)
        ENDIF
 1070 CONTINUE
      IF(NEDU.GT.0) THEN
        CALL LCMPUT(IPLIB,'ADDXSNAME',2*NEDU,3,IBASE(IADDXS))
      ENDIF
      CALL RLSARA(IBASE(IADDXS))
      IF(NNFI.GT.0) THEN
        CALL LCMPUT(IPLIB,'FISSIONNB',NBMIX*NNFI,1,NIFIS)
        CALL LCMPUT(IPLIB,'FISSIONNAMES',2*NNFI,3,ISONFI)
      ENDIF
C----
C  GET OLD NUMBER OF MIXTURES
C  REQUIRED FOR NUSIGF, CHI AND NFTOT
C----
      NOMIX=ISTATE(2)
      IF(NOMIX .GT. NBMIX) CALL XABORT
     >  ('LIBDEN: FINAL NUMBER OF MIXTURE LESS THAN INITIAL')
C----
C  READ SCATTERING CROSS SECTION MATRIX FOR ALL GROUPS
C  AND STORE INFORMATION FOR NON UPDATED MIXTURE
C----
      DO 2100 IG1=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IG1,NGROUP
        CALL LCMSIX(IPLIB,CGRPNM,1)
C----
C  READ MACROSCOPIC X-S FOR MIXTURES  NOT UPDATED
C  AND SAVE IN SCAT
C----
        LENGTH=1
        DO 2110 IL=1,NL
          IF(IPSC(IL).NE.0) THEN
            WRITE(CM,'(I2)') IL-1
            CALL LCMLEN(IPLIB,'SCAD'//CM,LENGTH,ITYLCM)
            IF(LENGTH.GT.0) THEN
              CALL LCMGET(IPLIB,'SCAD'//CM,SCAT)
              CALL LCMGET(IPLIB,'NJJD'//CM,NJJ)
              CALL LCMGET(IPLIB,'IJJD'//CM,IJJ)
              CALL LCMGET(IPLIB,'IPOD'//CM,IPOS)
              DO 2112 IMX=1,NOMIX
                IF(.NOT.MASK(IMX)) THEN
                  IPOSDE=IPOS(IMX)
                  IGFIN=IJJ(IMX)
                  IGDEB=IGFIN-NJJ(IMX)+1
                  DO 2113 IG2=IGFIN,IGDEB,-1
                    SCT(IMX,IG2,IL,IG1)=SCAT(IPOSDE)
                    IPOSDE=IPOSDE+1
 2113             CONTINUE
                ENDIF
 2112         CONTINUE
            ENDIF
          ENDIF
 2110   CONTINUE
        CALL LCMSIX(IPLIB,' ',2)
 2100 CONTINUE
C----
C  CHECK IF UPDATE REQUIRED IN GROUP IG1
C----
      DO 1100 IG1=1,NGROUP
        LOGL=MASKG(IG1)
        DO 1101 IL=1,NL
          DO 1102 IG2=1,NGROUP
            DO 1103 IMX=1,NBMIX
              LOGL=LOGL.OR.
     >            (MASKG(IG2).AND.(SCT(IMX,IG2,IL,IG1).NE.0.0))
 1103       CONTINUE
 1102     CONTINUE
 1101   CONTINUE
        IF(LOGL) THEN
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IG1,NGROUP
          CALL LCMSIX(IPLIB,CGRPNM,1)
C----
C  READ MACROSCOPIC X-S FOR MIXTURES AND GROUPS NOT UPDATED
C  AND SAVE FINAL X-S
C----
          LENGTH=1
          DO 1110 IL=1,NL
            IF(IPSC(IL).NE.0) THEN
              WRITE(CM,'(I2)') IL-1
C              IF(MASKK) THEN
C                IF(IL.GT.1)
C     >            CALL LCMLEN(IPLIB,'SCAD'//CM,LENGTH,ITYLCM)
C                IF(LENGTH.GT.0) THEN
C                  DO 1111 IMX=1,NBMIX
C                    IPOS(IMX)=-99
C 1111             CONTINUE
C                  CALL LCMGET(IPLIB,'SCAD'//CM,SCAT)
C                  CALL LCMGET(IPLIB,'NJJD'//CM,NJJ)
C                  CALL LCMGET(IPLIB,'IJJD'//CM,IJJ)
C                  CALL LCMGET(IPLIB,'IPOD'//CM,IPOS)
C                  DO 1112 IMX=1,NOMIX
C                    IF(.NOT.MASK(IMX)) THEN
C                      IPOSDE=IPOS(IMX)
C                      IF(IPOSDE.EQ.-99) GO TO 1112
C                      IGFIN=IJJ(IMX)
C                      IGDEB=IGFIN-NJJ(IMX)+1
C                      DO 1113 IG2=IGFIN,IGDEB,-1
C                        SCT(IMX,IG2,IL,IG1)=SCAT(IPOSDE)
C                        IPOSDE=IPOSDE+1
C 1113                 CONTINUE
C                    ENDIF
C 1112             CONTINUE
C                ENDIF
C              ENDIF
              IPOSDE=0
              DO 1114 IMX=1,NBMIX
                IPOS(IMX)=IPOSDE+1
                IGMIN=IG1
                IGMAX=IG1
                DO 1115 IG2=NGROUP,1,-1
                  IF(SCT(IMX,IG2,IL,IG1).NE.0.0) THEN
                    IGMIN=MIN(IGMIN,IG2)
                    IGMAX=MAX(IGMAX,IG2)
                  ENDIF
 1115           CONTINUE
                IJJ(IMX)=IGMAX
                IF(IGMAX.GT.IG1) THEN
                  IFGUPS(IMX,IL)=MIN(IFGUPS(IMX,IL),IG1)
                ENDIF
                NJJ(IMX)=IGMAX-IGMIN+1
                DO 1116 IG2=IGMAX,IGMIN,-1
                  IPOSDE=IPOSDE+1
                  SCAT(IPOSDE)=SCT(IMX,IG2,IL,IG1)
 1116           CONTINUE
                GAS(IMX)=SCAT(IPOS(IMX)+IJJ(IMX)-IG1)
 1114         CONTINUE
              CALL LCMPUT(IPLIB,'SCAD'//CM,IPOSDE,2,SCAT)
              CALL LCMPUT(IPLIB,'NJJD'//CM,NBMIX,1,NJJ)
              CALL LCMPUT(IPLIB,'IJJD'//CM,NBMIX,1,IJJ)
              CALL LCMPUT(IPLIB,'IPOD'//CM,NBMIX,1,IPOS)
              CALL LCMPUT(IPLIB,'SIGW'//CM,NBMIX,2,GAS)
              IPOSDE=0
              DO 1117 IMX=1,NBMIX
                IPOS(IMX)=IPOSDE+1
                IGMIN=IG1
                IGMAX=IG1
                DO 1118 IG2=NGROUP,1,-1
                  IF(SCT(IMX,IG1,IL,IG2).NE.0.0) THEN
                    IGMIN=MIN(IGMIN,IG2)
                    IGMAX=MAX(IGMAX,IG2)
                  ENDIF
 1118           CONTINUE
                IJJ(IMX)=IGMAX
                NJJ(IMX)=IGMAX-IGMIN+1
                DO 1119 IG2=IGMAX,IGMIN,-1
                  IPOSDE=IPOSDE+1
                  SCAT(IPOSDE)=SCT(IMX,IG1,IL,IG2)
 1119           CONTINUE
 1117         CONTINUE
              CALL LCMPUT(IPLIB,'SCAA'//CM,IPOSDE,2,SCAT)
              CALL LCMPUT(IPLIB,'NJJA'//CM,NBMIX,1,NJJ)
              CALL LCMPUT(IPLIB,'IJJA'//CM,NBMIX,1,IJJ)
              CALL LCMPUT(IPLIB,'IPOA'//CM,NBMIX,1,IPOS)
            ENDIF
 1110     CONTINUE
          IL=1
          IF(IPXS(IL).NE.0) THEN
            IF(MASKK) THEN
              CALL XDRSET(GAS,NBMIX,0.0)
              CALL LCMGET(IPLIB,'TOTAL',GAS)
              DO 1120 IMX=1,NOMIX
                IF(.NOT.MASK(IMX)) THEN
                  GXS(IMX,IL,IG1)=GAS(IMX)
                ENDIF
 1120         CONTINUE
            ENDIF
            CALL LCMPUT(IPLIB,'TOTAL',NBMIX,2,GXS(1,IL,IG1))
          ENDIF
          IL=IL+1
          IF(IPXS(IL).NE.0) THEN
            IF(MASKK) THEN
              CALL XDRSET(GAS,NBMIX,0.0)
              CALL LCMGET(IPLIB,'TRANC',GAS)
              DO 1121 IMX=1,NOMIX
                IF(.NOT.MASK(IMX)) THEN
                  GXS(IMX,IL,IG1)=GAS(IMX)
                ENDIF
 1121         CONTINUE
            ENDIF
            CALL LCMPUT(IPLIB,'TRANC',NBMIX,2,GXS(1,IL,IG1))
          ENDIF
          JL=IL
          IF(NEDU.NE.0) THEN
            DO 1130 IED=1,NED
              JL=JL+1
              IF(IPXS(JL).NE.0) THEN
                IF(MASKK) THEN
                  CALL LCMLEN(IPLIB,HVECT(IED),LENGTH,ITYLCM)
                  IF(LENGTH.NE.0) THEN
                    CALL XDRSET(GAS,NBMIX,0.0)
                    CALL LCMGET(IPLIB,HVECT(IED),GAS)
                    DO 1131 IMX=1,NOMIX
                      IF(.NOT.MASK(IMX)) THEN
                        GXS(IMX,JL,IG1)=GAS(IMX)
                      ENDIF
 1131               CONTINUE
                  ENDIF
                ENDIF
                CALL LCMPUT(IPLIB,HVECT(IED),NBMIX,2,GXS(1,JL,IG1))
              ENDIF
 1130       CONTINUE
          ENDIF
          JNFL=IL+NED
          JFTL=JNFL+NNFI
          JCHL=JFTL+NNFI
          IF(NNFI.NE.0) THEN
            CALL LCMLEN(IPLIB,'NUSIGF',LENGTH,ITYLCM)
            IF(MASKK.AND.LENGTH.NE.0) THEN
              CALL XDRSET(GAS,3*NBMIX*NNFI,0.0)
              ILCNF=0
              ILCFT=ILCNF+NNFI*NBMIX
              ILCCH=ILCFT+NNFI*NBMIX
              CALL LCMGET(IPLIB,'NUSIGF',GAS(ILCNF+1))
              CALL LCMGET(IPLIB,'NFTOT ',GAS(ILCFT+1))
              CALL LCMGET(IPLIB,'CHI   ',GAS(ILCCH+1))
              DO 1140 IFIS=1,NNFI
                JNFL=JNFL+1
                JFTL=JFTL+1
                JCHL=JCHL+1
                IF(IPXS(JNFL).NE.0) THEN
                  DO 1141 IMX=1,NOMIX
                    IF(.NOT.MASK(IMX)) THEN
                      GXS(IMX,JNFL,IG1)=GAS(ILCNF+IMX)
                    ENDIF
 1141             CONTINUE
                  DO 1142 IMX=1,NOMIX
                    IF(.NOT.MASK(IMX)) THEN
                      GXS(IMX,JFTL,IG1)=GAS(ILCFT+IMX)
                    ENDIF
 1142             CONTINUE
                  DO 1143 IMX=1,NOMIX
                    IF(.NOT.MASK(IMX)) THEN
                      GXS(IMX,JCHL,IG1)=GAS(ILCCH+IMX)
                    ENDIF
 1143             CONTINUE
                ENDIF
                ILCNF=ILCNF+NOMIX
                ILCFT=ILCFT+NOMIX
                ILCCH=ILCCH+NOMIX
 1140         CONTINUE
            ENDIF
            JNFL=IL+NED+1
            JFTL=JNFL+NNFI
            JCHL=JFTL+NNFI
            CALL LCMPUT(IPLIB,'NUSIGF',NBMIX*NNFI,2,
     >                  GXS(1,JNFL,IG1))
            CALL LCMPUT(IPLIB,'NFTOT ',NBMIX*NNFI,2,
     >                  GXS(1,JFTL,IG1))
            CALL LCMPUT(IPLIB,'CHI   ',NBMIX*NNFI,2,
     >                  GXS(1,JCHL,IG1))
          ENDIF
          CALL LCMSIX(IPLIB,' ',2)
        ENDIF
 1100 CONTINUE
      IF(MASKK) THEN
        ISTATE(2)=MAX(ISTATE(2),NBMIX)
        ISTATE(3)=MAX(ISTATE(3),NL)
        ISTATE(4)=MAX(ISTATE(4),NNFI)
        ISTATE(5)=MAX(ISTATE(5),NEDU)
        IF(ITRANC.NE.0) ISTATE(6)=ITRANC
      ELSE
        ISTATE(4)=NNFI
        ISTATE(5)=NEDU
        ISTATE(6)=ITRANC
        TEXT12='L_MACROLIB'
        READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
        CALL LCMPUT(IPLIB,'SIGNATURE',3,3,KCHAR)
      ENDIF
      ISTATE(7)=0
      CALL LCMPUT(IPLIB,'TIMESTAMP',3,2,TMPDAY)
      DO 1150 IL=1,NL
        DO 1151 IMX=1,NBMIX
          IF(IFGUPS(IMX,IL) .EQ. NGROUP+1) IFGUPS(IMX,IL)=1
 1151   CONTINUE
 1150 CONTINUE
      CALL LCMPUT(IPLIB,'FGWITHUPSCAT',NBMIX*NL,1,IFGUPS)
      CALL LCMPUT(IPLIB,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL LCMSIX(IPLIB,' ',2)
C----
C  FORMAT
C----
 6000 FORMAT(' WARNING IN LIBDEN FOR PERTURBATION'/
     >       ' EXTRAPOLATION BELOW PRETURBATION TABLES'/
     >       ' INITIAL TIME       = ',F15.6,' DAYS'/
     >       ' EXTRAPOLATION TIME = ',F15.6,' DAYS')
 6001 FORMAT(' WARNING IN LIBDEN FOR PERTURBATION'/
     >       ' EXTRAPOLATION ABOVE PRETURBATION TABLES'/
     >       ' FINAL TIME         = ',F15.6,' DAYS'/
     >       ' EXTRAPOLATION TIME = ',F15.6,' DAYS')
      RETURN
      END
