*DECK MACPXS
      SUBROUTINE MACPXS(IPMACR,NBMIX ,NGROUP,NANISO,NIFISS,MAXFIS,
     >                  ITRANC,LNEWXS,NEDMAC,
     >                  XSTOTL,XSTRAN,XSFISS,XSFIXE,XSSCAT,XSWORK,
     >                  INGSCT,IFGSCT,IPOSCT,ISCATA,XEFISS,IFGUPS)
C
C---------------------------  MACPXS  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : MACPXS
C       USE      : TRANSFER X-S ON THE MACROLIB
C       MODIFIED : 2000-01-20 (T. COURAU)
C                  91-01-24
C       AUTHOR   : G.MARLEAU
C
C   2- ROUTINE PARAMETERS:
C    INPUT
C       IPMACR   : POINTER TO THE MACROLIB            I
C       NBMIX    : NUMBER OF MIXTURES                 I
C       NGROUP   : NUMBER OF ENERGY GROUPS            I
C       NANISO   : MAXIMUM LEGENDRE ORDER             I
C                  = 1 ISOTROPIC COLLISION
C                  = 2 LINEARLY ANISOTROPIC COLLISION
C       NIFISS   : NUMBER OF FISSILE ISOTOPES         I
C       MAXFIS   : MAX(1,NIFISS)                      I
C       ITRANC   : TRANSPORT CORRECTION OPTION        I
C       LNEWXS   : CHECK CHANGE IN X-S                L(NBXS)
C
C       NEDMAC   : NUMBER OF MACRO EDIT X-SECTIONS    I
C       XSTOTL   : TOTAL CROSS SECTION OF MIXTURE     R(NBMIX,NGROUP)
C       XSTRAN   : TRANSPORT CORRECTION XS OF MIXTURE R(NBMIX,NGROUP)
C       XSFISS   : FISSION CROSS SECTION OF MIXTURE   R(NBMIX,
C                                                    MAXFIS,NGROUP,3)
C       XSFIXE   : FIXE SOURCES                       R(NBMIX,
C                                                      NGROUP,3)
C       XSSCAT   : SCATT X-SECTION OF MIXTURE/GROUP   R(NGROUP,NBMIX,
C                                                       NANISO,NGROUP)
C       XSWORK   : WORK CROSS-SECTION VECTOR          R(NBMIX
C                                                       *(1+NGROUP))
C       INGSCT   : NB OF SCATTERING GROUP FOR  XS     I(NBMIX)
C       IFGSCT   : FIRST SCATTERING GROUP FOR  XS     I(NBMIX)
C       IPOSCT   : MATERIAL POSITION IN SCATTERING    I(NBMIX)
C       ISCATA   : CHECK FOR SCATTERING ANISOTROPY    I(NANISO)
C       XEFISS   : AVERAGE ENERGY PER FISSION         R(NBMIX,MAXFIS)
C       IFGUPS   : FIRST GROUP WITH UP-SCATTERING     I(NBMIX,NANISO)
C
C---------------------------   MACPXS  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,NBXS,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NBXS=10,ILCMUP=1,ILCMDN=2,NAMSBR='MACPXS')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPMACR,NBMIX,NGROUP,NANISO,NIFISS,MAXFIS,ITRANC,
     >            NEDMAC
      INTEGER     INGSCT(NBMIX),IFGSCT(NBMIX),IPOSCT(NBMIX),
     >            ISCATA(NANISO),IFGUPS(NBMIX,NANISO)
      LOGICAL     LNEWXS(NBXS)
      REAL        XSTOTL(NBMIX,NGROUP),XSTRAN(NBMIX,NGROUP),
     >            XSFISS(NBMIX,MAXFIS,NGROUP,3),
     >            XSFIXE(NBMIX,NGROUP,3),
     >            XSSCAT(NGROUP,NBMIX,NANISO,NGROUP),
     >            XSWORK(NBMIX*(NGROUP+1)),
     >            XEFISS(NBMIX,MAXFIS)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IFISN,IFISM
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12,CANISO*2,CNAFIS*8
      INTEGER     ILCMLN,ILCMTY,IANIS,NSVFIS,IFIS,JFIN,JFIM,ITC,IGR,
     >            NELEM,IMIX,JGR,J1,J2
      DOUBLE PRECISION DBXS
C----
C  GET NUMBER OF MACRO EDIT X-SECTIONS
C  INITIALIZE FIRST GROUP WITH UP-SCATTERING
C----
      CALL LCMLEN(IPMACR,'ADDXSNAME',ILCMLN,ILCMTY)
      NEDMAC=ILCMLN/2
      CALL LCMLEN(IPMACR,'FGWITHUPSCAT',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0 .AND. ILCMLN .LE. NBMIX*NANISO ) THEN
        CALL LCMGET(IPMACR,'FGWITHUPSCAT',IFGUPS)
        IF(LNEWXS(5)) THEN
          DO 100 IANIS=1,NANISO
            IF(ISCATA(IANIS) .EQ. 2) THEN
              CALL XDRSET(IFGUPS(1,IANIS),NBMIX,NGROUP+1)
            ENDIF
 100      CONTINUE
        ENDIF
      ELSE
        CALL XDRSET(IFGUPS,NBMIX*NANISO,NGROUP+1)
      ENDIF
C----
C  SAVE DUMMY FISSILE ISOTOPE NAMES
C----
      IF(NIFISS .GT. 0 .AND. LNEWXS(2) ) THEN
        CALL LCMLEN(IPMACR,'FISSIONNAMES',ILCMLN,ILCMTY)
        NSVFIS=ILCMLN/2
        IF(NSVFIS .LT. NIFISS) THEN
          CALL SETARA(IBASE,2*NIFISS,IFISN)
          CALL SETARA(IBASE,NBMIX*NIFISS,IFISM)
          IF(NSVFIS .EQ. 0) THEN
            JFIN=IFISN
            JFIM=IFISM
          ELSE
            CALL LCMGET(IPMACR,'FISSIONNAMES',IBASE(IFISN))
            CALL LCMGET(IPMACR,'FISSIONNB',IBASE(IFISM))
            JFIM=IFISM+NBMIX*NSVFIS
            JFIN=IFISN+2*NSVFIS
          ENDIF
          DO 110 IFIS=NSVFIS+1,NIFISS
            WRITE(CNAFIS,'(A5,I3)') 'MACFI',IFIS
            READ(CNAFIS,'(2A4)') (IBASE(ITC),ITC=JFIN,JFIN+1)
            JFIN=JFIN+2
 110      CONTINUE
          CALL XDRSET(IBASE(JFIM),NBMIX*(NIFISS-NSVFIS),-1)
          CALL LCMPUT(IPMACR,'FISSIONNB',NIFISS*NBMIX,1,IBASE(IFISM))
          CALL LCMPUT(IPMACR,'FISSIONNAMES',2*NIFISS,3,IBASE(IFISN))
          CALL RLSARA(IBASE(IFISM))
          CALL RLSARA(IBASE(IFISN))
        ENDIF
      ENDIF
      DO 120 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
C----
C  PUT TOTAL, AND FIXE ON THE MACROLIB IF MODIFIED
C----
        IF(LNEWXS(1))
     >    CALL LCMPUT(IPMACR,'TOTAL',NBMIX,2,XSTOTL(1,IGR))
        IF(LNEWXS(6) .AND. ITRANC .GT. 0)
     >    CALL LCMPUT(IPMACR,'TRANC',NBMIX,2,XSTRAN(1,IGR))
        IF(LNEWXS(3))
     >    CALL LCMPUT(IPMACR,'FIXE' ,NBMIX,2,XSFIXE(1,IGR,1))
        IF(LNEWXS(7))
     >    CALL LCMPUT(IPMACR,'FIXA' ,NBMIX,2,XSFIXE(1,IGR,2))
        IF(LNEWXS(8))
     >    CALL LCMPUT(IPMACR,'FIXG' ,NBMIX,2,XSFIXE(1,IGR,3))
C----
C  PUT CHI AND NUSIGF ON THE MACROLIB IF MODIFIED
C----
        IF(NIFISS .GT. 0) THEN
          IF(LNEWXS(2))
     >      CALL LCMPUT(IPMACR,'NUSIGF',NBMIX*NIFISS,2,
     >                  XSFISS(1,1,IGR,1))
          IF(LNEWXS(9))
     >      CALL LCMPUT(IPMACR,'NFTOT ',NBMIX*NIFISS,2,
     >                  XSFISS(1,1,IGR,2))
          IF(LNEWXS(4))
     >      CALL LCMPUT(IPMACR,'CHI   ',NBMIX*NIFISS,2,
     >                  XSFISS(1,1,IGR,3))
        ENDIF
C----
C  STORE ADJOINT CROSS SECTIONS ON THE MACROLIB IF REQUIRED
C  PUT CHI(ADJOINT)=NUSIGF DIRECT
C  NUSIGF(ADJOINT) =CHI DIRECT
C----
C----
C  COMPRESS AND PUT SCATT ON THE MACROLIB IF MODIFIED
C----
        DO 130 IANIS=1,NANISO
          WRITE(CANISO,'(I2)') IANIS-1
          IF(LNEWXS(5) .AND. ISCATA(IANIS) .EQ. 2) THEN
            NELEM=0
            DO 140 IMIX=1,NBMIX
              J2=IGR
              J1=IGR
              DO 141 JGR=1,NGROUP
                IF(XSSCAT(JGR,IMIX,IANIS,IGR) .NE. 0.0) THEN
                  J2=MAX(J2,JGR)
                  J1=MIN(J1,JGR)
                ENDIF
 141          CONTINUE
              INGSCT(IMIX)=J2-J1+1
              IFGSCT(IMIX)=J2
              IF(J2 .GT. IGR) THEN
                IFGUPS(IMIX,IANIS)=MIN(IFGUPS(IMIX,IANIS),IGR)
              ENDIF
              IPOSCT(IMIX)=NELEM+1
              DO 142 JGR=J2,J1,-1
                NELEM=NELEM+1
                XSWORK(NBMIX+NELEM)=XSSCAT(JGR,IMIX,IANIS,IGR)
 142          CONTINUE
C----
C  STORE DIAGONAL ELEMENTS OF SCATTERING MATRIX
C  AND TOTAL SCATTERING OUT OF GROUP
C----
              XSWORK(IMIX)=XSSCAT(IGR,IMIX,IANIS,IGR)
 140        CONTINUE
            CALL LCMPUT(IPMACR,'SIGW'//CANISO,NBMIX,2,XSWORK)
            CALL LCMPUT(IPMACR,'NJJD'//CANISO,NBMIX,1,INGSCT)
            CALL LCMPUT(IPMACR,'IJJD'//CANISO,NBMIX,1,IFGSCT)
            CALL LCMPUT(IPMACR,'IPOD'//CANISO,NBMIX,1,IPOSCT)
            CALL LCMPUT(IPMACR,'SCAD'//CANISO,NELEM,2,XSWORK(NBMIX+1))
            NELEM=0
            DO 150 IMIX=1,NBMIX
              J2=IGR
              J1=IGR
              DO 151 JGR=1,NGROUP
                IF(XSSCAT(IGR,IMIX,IANIS,JGR) .NE. 0.0) THEN
                  J2=MAX(J2,JGR)
                  J1=MIN(J1,JGR)
                ENDIF
 151          CONTINUE
              INGSCT(IMIX)=J2-J1+1
              IFGSCT(IMIX)=J2
              IPOSCT(IMIX)=NELEM+1
              DBXS=0.0D0
              DO 152 JGR=J2,J1,-1
                NELEM=NELEM+1
                XSWORK(NBMIX+NELEM)=XSSCAT(IGR,IMIX,IANIS,JGR)
                DBXS=DBXS+DBLE(XSSCAT(IGR,IMIX,IANIS,JGR))
 152          CONTINUE
              XSWORK(IMIX)=REAL(DBXS)
C----
C  STORE DIAGONAL ELEMENTS OF SCATTERING MATRIX
C  AND TOTAL SCATTERING OUT OF GROUP
C----
 150        CONTINUE
            IF(IANIS .EQ. 2 .AND. ITRANC .EQ. 1)
     >      CALL LCMPUT(IPMACR,'TRANC',NBMIX,2,XSWORK)
            CALL LCMPUT(IPMACR,'NJJA'//CANISO,NBMIX,1,INGSCT)
            CALL LCMPUT(IPMACR,'IJJA'//CANISO,NBMIX,1,IFGSCT)
            CALL LCMPUT(IPMACR,'IPOA'//CANISO,NBMIX,1,IPOSCT)
            CALL LCMPUT(IPMACR,'SCAA'//CANISO,NELEM,2,XSWORK(NBMIX+1))
          ENDIF
 130    CONTINUE
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 120  CONTINUE
      IF(LNEWXS(10)) 
     >  CALL LCMPUT(IPMACR,'EFISS',NBMIX*NIFISS,2,XEFISS)
      IF(LNEWXS(5)) THEN
        CALL LCMPUT(IPMACR,'FGWITHUPSCAT',NBMIX*NANISO,1,IFGUPS)
      ENDIF
      RETURN
      END
