*DECK QIJRHL
      SUBROUTINE QIJRHL(NREG,NSNEG,LGRP,NGRP,SIGTAL,PROB,
     >                  CHI,WFSP,WEIG,WFSPAD)
C
C-------------------------    QIJRHL    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : QIJRHL
C     LEVEL    : 4 (CALLED BY 'XL3TRK')
C     USE      : NON-LINEAR NORMALIZATION OF COLLISION PROBS
C     AUTHOR   : R. ROY (94-04-18)
C     REFERENCE: 'NORMALIZATION TECHNIQUES FOR CP MATRICES',
C                 R.ROY AND G.MARLEAU,
C                 CONF/PHYSOR-90, MARSEILLE/FRANCE, V 2, P IX-40 (1990).
C                 'HELIOS: ANGULARLY DEPENDENT COLLISION PROBABILITIES'
C                 E.A. VILLARINO, R.J.J.STAMM'LER
C                 AND A.A.FERRI AND J.J.CASAL
C                 Nucl.Sci.Eng. 112,16-31, 1992.
C
C 2- PARAMETERS:
C  INPUT
C     NREG    : # OF ZONES FOR GEOMETRY.             I
C     NSNEG   : # OF SURFACES FOR GEOMETRY.          I
C     LGRP    : LIST OF GROUPS.                      I(NGRP)
C     NGRP    : # OF GROUPS.                         I
C     SIGTAL  : ALBEDO-SIGT VECTOR                   R(NGRP,-NSOUT:NREG)
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,NPRB)
C               NPRB=(NSOUT+NREG+1)*(NSOUT+NREG+2)/2
C  OUTPUT
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,NPRB)
C  WORK
C     WEIG    : ADDITIVE WEIGHT                      R(NGRP,-NSOUT:NREG,
C     WFSP    : WORK AREA                            R(NGRP)
C     WFSPAD  : WORK AREA                            R(NGRP)
C     CHI     : WORK AREA                            R(NGRP,-NSOUT:NREG)
C
C
C-------------------------    QIJRHL    -------------------------------
C
      IMPLICIT   NONE
      INTEGER    NREG,NSOUT,NGRP,IUNOUT,NITMAX,NIT,NSNEG,
     >           IR,JR,JG,IG,IP,IPRB,IND,I,J,CPTLB,CPTAC,CTOT,
     >           NSURC,NSURM,NVOLC,NVOLM,IPRT
      INTEGER    LGRP(NGRP)
      REAL       SIGTAL(NGRP,NSNEG:NREG)
      LOGICAL    NOTCON
      DOUBLE PRECISION PROB(NGRP,*),WEIG(NGRP,NSNEG:NREG,3),
     >           CHI(NGRP,NSNEG:NREG),WFSPAD(NGRP),WFSP(NGRP),
     >           EPSCON,TOTCON,TOTALL,NOM,DENOM,R1,R2,MU
      PARAMETER (IUNOUT=6, EPSCON=1.0D-12, NITMAX=200, IPRT=0)
C
C----- INTRINSIC FUNCTION FOR POSITION IN CONDENSE PIJ MATRIX
C
      IND(I,J)=(MAX(I+NSOUT+1,J+NSOUT+1)*
     >         (MAX(I+NSOUT+1,J+NSOUT+1)-1))/2
     >         +MIN(I+NSOUT+1,J+NSOUT+1)
C
      NSOUT= -NSNEG
      NOTCON= .FALSE.
      CPTLB = 3
      CPTAC = 3
      CTOT = CPTAC+CPTLB
C
C     INITIALISATION OF WEIGHTS
      DO 60 IR=-NSOUT, NREG
         DO 55 IG=1, NGRP
            WEIG(IG,IR,1)=0.0D0
            WEIG(IG,IR,2)=0.5D0
            WEIG(IG,IR,3)=0.5D0
   55    CONTINUE
   60 CONTINUE
      DO 50 IR=-NSOUT, NREG
         DO 61 IG=1, NGRP
           CHI(IG,IR)= 1.0D0
           IF( IR.GE.0.AND.SIGTAL(IG,IR).EQ.0.0 )THEN
              CHI(IG,IR)= 0.0D0
           ENDIF
   61    CONTINUE
   50 CONTINUE
C
C
C     MAIN ITERATION LOOP
      IF( IPRT.GT.1 )THEN
         WRITE(IUNOUT,'(A24)')
     >          'ITER.     MU      ERROR '
      ENDIF
      DO 110 NIT=1,NITMAX
C
         DO 220 IR= -NSOUT, NREG
            DO 205 IG=1, NGRP
               WFSPAD(IG)= PROB(IG,IND(IR,0))
     >                   + CHI(IG,IR)*PROB(IG,IND(IR,IR))*WEIG(IG,IR,3)
               WFSP(IG) = CHI(IG,IR)*PROB(IG,IND(IR,IR))
  205       CONTINUE
            DO 200 JR=-NSOUT, NREG
               DO 206 IG=1, NGRP
                  WFSPAD(IG)=WFSPAD(IG)-CHI(IG,JR)
     >                      *WEIG(IG,JR,3)*PROB(IG,IND(IR,JR))
                  WFSP(IG)=WFSP(IG)
     >                    +CHI(IG,JR)*PROB(IG,IND(IR,JR))
  206          CONTINUE
  200       CONTINUE
            DO 208 IG=1, NGRP
               WEIG(IG,IR,3)=WFSPAD(IG)/WFSP(IG)
  208       CONTINUE
  220    CONTINUE
C
C        ACCELERATION TECHNIQUE
         IF(  MOD(NIT-1,CTOT).GE.CPTAC )THEN
            NOM   = 0.0D0
            DENOM = 0.0D0
            DO 10 IR=-NSOUT, NREG
               DO  9 IG= 1, NGRP
                  R1= WEIG(IG,IR,2) - WEIG(IG,IR,1)
                  R2= WEIG(IG,IR,3) - WEIG(IG,IR,2)
                  NOM = NOM + R1*(R2-R1)
                  DENOM = DENOM + (R2-R1)*(R2-R1)
    9          CONTINUE
   10       CONTINUE
            MU = - NOM / DENOM
            IF( MU.GT.2.0D0 .OR. MU.LT.0.0D0 )CALL XABORT('QIJRHL: '
     >         //'PROBLEM OF ACCELERATION')
            DO 20 IR=-NSOUT, NREG
               DO 19 IG= 1, NGRP
                  WEIG(IG,IR,3) = WEIG(IG,IR,2) + MU *
     >                           (WEIG(IG,IR,3) - WEIG(IG,IR,2))
                  WEIG(IG,IR,2) = WEIG(IG,IR,1) + MU *
     >                           (WEIG(IG,IR,2) - WEIG(IG,IR,1))
   19          CONTINUE
   20       CONTINUE
         ELSE
            MU = 1.0D0
         ENDIF
C
C        CALCULATIONS OF SQUARE DISTANCE BETWEEN 2 ITERATIONS
C        AND UPDATING THE SOLUTION
         TOTALL = 0.0D0
         DO 100 IR=-NSOUT, NREG
            TOTCON = 0.0D0
            DO  95 IG= 1, NGRP
               TOTCON= TOTCON + (WEIG(IG,IR,3)-WEIG(IG,IR,2))**2
               WEIG(IG,IR,1)= WEIG(IG,IR,2)
               WEIG(IG,IR,2)= WEIG(IG,IR,3)
   95       CONTINUE
            TOTALL= TOTALL + TOTCON
  100    CONTINUE
         TOTALL= TOTALL/DBLE(NGRP)
         IF( IPRT.GT.1 )THEN
            WRITE(IUNOUT,'(I3,F11.5,E15.5)') NIT,MU,TOTALL
         ENDIF
C
C        CONVERGENCE TEST
         IF( TOTALL.LT.EPSCON )GO TO 120
C
  110 CONTINUE
      NOTCON=.TRUE.
      WRITE(IUNOUT,'(35H QIJRHL: WEIGHTS NOT CONVERGED          )')
  120 CONTINUE
C
C     RENORMALIZE "PIJ" SYMMETRIC MATRIX
      IPRB = 0
      DO 240 IR   = -NSOUT, NREG
         DO 230 JR= -NSOUT, IR
            IPRB= IPRB+1
            IF( IR.NE.0.AND.JR.NE.0 )THEN
               DO 225 IG=1, NGRP
                  PROB(IG,IPRB)=PROB(IG,IPRB)
     >                         *(WEIG(IG,IR,1)+WEIG(IG,JR,1))
  225          CONTINUE
            ENDIF
  230    CONTINUE
  240 CONTINUE
C
C     PRINT WEIGHT FACTORS IF THERE IS A PROBLEM...
      IF( NOTCON )THEN
         DO 350 IG=1, NGRP
         JG= LGRP(IG)
         WRITE(IUNOUT,*) 'GROUPE: ',JG
         WRITE(IUNOUT,'(30H0 SURFACE WEIGHTS FACTORS                /)')
         NSURC = -1
         DO 300 IP  = 1, (9 +NSOUT) / 10
            NSURM= MAX( -NSOUT, NSURC-9 )
            WRITE(IUNOUT,'(10X,10( A5,    I6)/)')
     >                     (' SUR ',-IR,IR= NSURC, NSURM, -1)
            WRITE(IUNOUT,'(10H WEIGHT   ,10F11.5)')
     >                   (WEIG(IG,IR,1),IR=NSURC,NSURM,-1)
            NSURC = NSURC - 10
 300     CONTINUE
         WRITE(IUNOUT,'(30H0  VOLUME WEIGHTS FACTORS                /)')
         NVOLC =  1
         DO 310 IP  = 1, (9 + NREG) / 10
            NVOLM= MIN( NREG, NVOLC+9 )
            WRITE(IUNOUT,'(10X,10( A5 ,  I6)/)')
     >                  (' VOL ',IR,IR=NVOLC,NVOLM, 1)
            WRITE(IUNOUT,'(10H WEIGHT   ,10F11.5)')
     >                   (WEIG(IG,IR,1),IR=NVOLC,NVOLM, 1)
            NVOLC = NVOLC + 10
 310     CONTINUE
 350     CONTINUE
      ENDIF
C
      RETURN
 8002 FORMAT(//' S U R F / S U R F    C O N T R I B U T I O N S'//
     >9X,'BEGIN S',6X,'END S ',11X,'CIJ.    ',11X, 'WSPACE  ',
     >                         11X,'PROBS.  ')
 8003 FORMAT(6X,I10,5X,I10,5X,1P,E15.7,5X,E15.7,5X,E15.7 )
 9000 FORMAT(//' F U N C T I O N     V A L U E S'//
     >9X,'VOL/SUR',6X,'VALUE')
 9001 FORMAT(6X,I10,5X,F10.4)
 9004 FORMAT(//' W E I G H T E D     V A L U E S'//
     >9X,'VOL/SUR',6X,'VALUE')
 9005 FORMAT(6X,I10,5X,F10.4)
      END
