*DECK QIJRNL
      SUBROUTINE QIJRNL(NREG,NSNEG,LGRP,NGRP,SIGTAL,PROB,CIJ,
     >                  WSPACE,WFSP,IDL,WEIG,WFSPAD,T)
C
C-------------------------    QIJRNL    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : QIJRNL
C     LEVEL    : 4 (CALLED BY 'XL3TRK')
C     USE      : NON-LINEAR NORMALIZATION OF COLLISION PROBS
C     AUTHOR   : R. ROY (94-04-18)
C     REFERENCE: 'NORMALIZATION TECHNIQUES FOR CP MATRICES',
C                 R.ROY AND G.MARLEAU,
C                 CONF/PHYSOR-90, MARSEILLE/FRANCE, V 2, P IX-40 (1990).
C
C 2- PARAMETERS:
C  INPUT
C     NREG    : # OF ZONES FOR GEOMETRY.             I
C     NSNEG   : # OF SURFACES FOR GEOMETRY.          I
C     LGRP    : LIST OF GROUPS.                      I(NGRP)
C     NGRP    : # OF GROUPS.                         I
C     SIGTAL  : ALBEDO-SIGT VECTOR                   R(NGRP,-NSOUT:NREG)
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,NPRB)
C               NPRB=(NSOUT+NREG+1)*(NSOUT+NREG+2)/2
C  OUTPUT
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,NPRB)
C  WORK
C     CIJ     : MODIFIED CP PROB MATRIX              R(NGRP,NPRB)
C     WSPACE  : NON-LINEAR SYSTEM MATRIX             R(NGRP,NPRB)
C     WFSP    : NON LINEAR SYSTEM SOLUTION           R(NGRP,-NSOUT:NREG)
C     IDL     : WSPACE DIAGONAL LOCATION             I(-NSOUT:NREG)
C     WEIG    : NON-LINEAR WEIGHT                    R(NGRP,-NSOUT:NREG)
C     T       : WORK AREA                            R(2*NGRP)
C
C  3-INTERNAL PARAMETERS
C     EPSCON  : CONVERGENCE CRITERIA = 1.0D-12
C     NITMAX  : MAXIMUM NUMBER OF ITERATIONS = 20
C
C-------------------------    QIJRNL    -------------------------------
C
      IMPLICIT   NONE
      INTEGER    NREG,NSOUT,NGRP,IDL(*),IUNOUT,NITMAX,NIT,
     >           NUNKNO,IPRB,IPRF,IVOL,IDIA,IUNK,JUNK,IR,JR,IG,JG,NSNEG,
     >           NSURC,NSURM,NVOLC,NVOLM,IP,IPRBO
      INTEGER    LGRP(NGRP)
      REAL       SIGTAL(NGRP,NSNEG:NREG)
      LOGICAL    NOTCON
      DOUBLE PRECISION PROB(NGRP,*),T(NGRP,2),
     >           WFSP(NGRP,NSNEG:NREG),WEIG(NGRP,NSNEG:NREG),
     >           CIJ(NGRP,*),WSPACE(NGRP,*),EPSCON,TOTCON,TOTALL,
     >           WFSPAD(NGRP)
      PARAMETER (IUNOUT=6, EPSCON=1.0D-12, NITMAX=20)
      NSOUT= -NSNEG
      NOTCON= .FALSE.
C
C     CHARGE MATRIX "CIJ"
      NUNKNO=NREG+NSOUT+1
      IPRB= 0
      IUNK= 0
      IVOL= NSOUT*(NSOUT+1)/2
      DO 5 IG=1, NGRP
      DO 20 IR   = -NSOUT, NREG
         IUNK= IUNK+1
         IF( IR.LT.0.OR.SIGTAL(IG,IR).GT.0.0 )THEN
            DO 10 JR= -NSOUT, IR-1
               IPRB= IPRB+1
               IF( JR.LT.0.OR.SIGTAL(IG,JR).GT.0.0 )THEN
                  CIJ(IG,IPRB)= PROB(IG,IPRB)
               ELSE
                  CIJ(IG,IPRB)= 0.0D0
               ENDIF
   10       CONTINUE
         ELSE
            DO 15 JR= -NSOUT, IR-1
               IPRB= IPRB+1
               CIJ(IG,IPRB)= 0.0D0
   15       CONTINUE
         ENDIF
         IPRB= IPRB+1
         IDL(IUNK)= IPRB
         IF( IR.LT.0 )THEN
            IVOL= IVOL+1
            CIJ(IG,IPRB)= PROB(IG,IPRB)
         ELSEIF( IR.GT.0 )THEN
            IVOL= IVOL+IUNK-1
            IF( SIGTAL(IG,IR).GT.0.0 )THEN
               CIJ(IG,IPRB)= PROB(IG,IPRB)
            ELSE
               CIJ(IG,IPRB)= PROB(IG,IVOL)
            ENDIF
         ELSE
            IVOL= IVOL+1
            CIJ(IG,IPRB)= 1.0D0
         ENDIF
   20 CONTINUE
    5 CONTINUE
C
C     COPY MATRIX "CIJ" IN THE "WSPACE" ARRAY FOR INVERSION
C          AND ADD TO THE DIAGONAL ALL TERMS OF A LINE
      IPRB= 0
      IUNK= 0
      IDIA= 0
      DO 50 IR   = -NSOUT, NREG
         IUNK= IUNK+1
         IDIA= IDIA+IUNK
         DO 25 IG=1, NGRP
            WSPACE(IG,IDIA)= CIJ(IG,IDIA) + CIJ(IG,IDIA)
   25    CONTINUE
         DO 30 JR= -NSOUT, IR-1
            IPRB= IPRB+1
            DO 26 IG=1, NGRP
               WSPACE(IG,IPRB)= CIJ(IG,IPRB)
               WSPACE(IG,IDIA)= WSPACE(IG,IDIA) + CIJ(IG,IPRB)
   26       CONTINUE
   30    CONTINUE
         IPRB= IPRB+1
         IPRF= IPRB
         JUNK= IUNK
         DO 40 JR=  IR+1 , NREG
            IPRF= IPRF+JUNK
            JUNK= JUNK+1
            DO 35 IG=1, NGRP
               WSPACE(IG,IDIA)= WSPACE(IG,IDIA) + CIJ(IG,IPRF)
   35       CONTINUE
   40    CONTINUE
   50 CONTINUE
C
C     INVERSION OF THE INITIAL SYSTEM JACOBIAN MATRICES
      CALL ALDLFV(NGRP,NUNKNO,WSPACE,IDL,T)
C
C     INITIALISATION OF WEIGHTS
      DO 60 IR=-NSOUT, NREG
         DO 55 IG=1, NGRP
            WEIG(IG,IR)=1.0D0
   55    CONTINUE
   60 CONTINUE
      DO 61 IG=1, NGRP
         WEIG(IG,0)= 0.0D0
   61 CONTINUE
C
C     THE NON-LINEAR SYSTEM FOR WEIGHTS IS:
C      F1(W1, W2, ... WN)= W1*(W1*C11+W2*C12+ ... +WN*C1N) - TRUE1
C      F2(W1, W2, ... WN)= W2*(W1*C21+W2*C22+ ... +WN*C2N) - TRUE2
C      ...
C      FN(W1, W2, ... WN)= WN*(W1*CN1+W2*CN2+ ... +WN*CNN) - TRUEN
C     FORMING THE SYSTEM USING WEIGHTS "WEIG" & CONTRIBUTIONS "CIJ"
C
C     MAIN ITERATION LOOP
      DO 110 NIT=1,NITMAX
C
         IPRB= 0
         IUNK= 0
         IVOL= NSOUT*(NSOUT+1)/2
         DO 90 IR=-NSOUT, NREG
            IF( IR.LE.0 )THEN
               IVOL= IVOL+1
            ELSE
               IVOL= IVOL+IUNK
            ENDIF
            IUNK= IUNK+1
            DO 63 IG=1, NGRP
               WFSPAD(IG)= 0.0D0
   63       CONTINUE
            DO 70 JR=-NSOUT, IR
               IPRB= IPRB+1
               DO 65 IG=1, NGRP
                  WFSPAD(IG)=WFSPAD(IG)+WEIG(IG,JR)*CIJ(IG,IPRB)
   65          CONTINUE
   70       CONTINUE
            IPRF= IPRB
            JUNK= IUNK
            DO 80 JR= IR+1 , NREG
               IPRF= IPRF+JUNK
               JUNK= JUNK+1
               DO 75 IG=1, NGRP
                  WFSPAD(IG)=WFSPAD(IG)+WEIG(IG,JR)*CIJ(IG,IPRF)
   75          CONTINUE
   80       CONTINUE
            DO 85 IG=1, NGRP
               WFSP(IG,IR)=WEIG(IG,IR)*WFSPAD(IG)-PROB(IG,IVOL)
   85       CONTINUE
   90    CONTINUE
         CALL ALDLSV(NGRP,NUNKNO,IDL,WSPACE,WFSP,T)
C
C        CALCULATIONS OF SQUARE DISTANCE BETWEEN 2 ITERATIONS
C        AND UPDATING THE SOLUTION
         TOTALL = 0.0D0
         DO  95 IG= 1, NGRP
            TOTCON = 0.0D0
            DO 100 IR=-NSOUT, NREG
               TOTCON= TOTCON + WFSP(IG,IR)**2
               WEIG(IG,IR)= WEIG(IG,IR) - WFSP(IG,IR)
  100       CONTINUE
            TOTALL= TOTALL + TOTCON
   95    CONTINUE
         TOTALL= TOTALL/DBLE(NGRP)
C
C        CONVERGENCE TEST
         IF( TOTALL.LT.EPSCON )GO TO 120
C
  110 CONTINUE
      NOTCON=.TRUE.
      WRITE(IUNOUT,'(35H QIJRNL: WEIGHTS NOT CONVERGED          )')
  120 CONTINUE
C
C     RECOMPUTE WEIGTHS FOR VOID REGIONS
      IPRB= (NSOUT+1)*(NSOUT+2)/2
      IVOL= IPRB
      IPRBO= IPRB
      IUNK= NSOUT+1
      DO 205 IG=1, NGRP
      DO 220 IR= 1, NREG
         IVOL= IVOL+IUNK
         IUNK= IUNK+1
         IF( SIGTAL(IG,IR).EQ.0.0 )THEN
            WFSPAD(IG)= 0.0D0
            DO 200 JR=-NSOUT, IR
               IPRB= IPRB+1
               IF( JR.LT.0.OR.SIGTAL(IG,JR).GT.0.0 )THEN
                 WFSPAD(IG)=WFSPAD(IG)+WEIG(IG,JR)*PROB(IG,IPRB)
               ENDIF
  200       CONTINUE
            IPRF= IPRB
            JUNK= IUNK
            DO 210 JR= IR+1 , NREG
               IPRF= IPRF+JUNK
               JUNK= JUNK+1
               IF( JR.LT.0.OR.SIGTAL(IG,JR).GT.0.0 )THEN
                 WFSPAD(IG)=WFSPAD(IG)+WEIG(IG,JR)*PROB(IG,IPRF)
               ENDIF
  210       CONTINUE
               WEIG(IG,IR)=PROB(IG,IVOL)/WFSPAD(IG)
         ELSE
            IPRB= IPRB+IUNK
         ENDIF
  220 CONTINUE
  205 CONTINUE
C
C     RENORMALIZE "PIJ" SYMMETRIC MATRIX
      IPRB = 0
      DO 240 IR   = -NSOUT, NREG
         DO 230 JR= -NSOUT, IR
            IPRB= IPRB+1
            IF( IR.NE.0.AND.JR.NE.0 )THEN
               DO 225 IG=1, NGRP
                  PROB(IG,IPRB)=PROB(IG,IPRB)*WEIG(IG,IR)*WEIG(IG,JR)
  225          CONTINUE
            ENDIF
  230    CONTINUE
  240 CONTINUE
C
C     PRINT WEIGHT FACTORS IF THERE IS A PROBLEM...
      IF( NOTCON )THEN
         DO 350 IG=1, NGRP
         JG= LGRP(IG)
         WRITE(IUNOUT,*) 'GROUPE: ',JG
         WRITE(IUNOUT,'(30H0 SURFACE WEIGHTS FACTORS                /)')
         NSURC = -1
         DO 300 IP  = 1, (9 +NSOUT) / 10
            NSURM= MAX( -NSOUT, NSURC-9 )
            WRITE(IUNOUT,'(10X,10( A5,    I6)/)')
     >                     (' SUR ',-IR,IR= NSURC, NSURM, -1)
            WRITE(IUNOUT,'(10H WEIGHT   ,10F11.5)')
     >                   (WEIG(IG,IR),IR=NSURC,NSURM,-1)
            NSURC = NSURC - 10
 300     CONTINUE
         WRITE(IUNOUT,'(30H0  VOLUME WEIGHTS FACTORS                /)')
         NVOLC =  1
         DO 310 IP  = 1, (9 + NREG) / 10
            NVOLM= MIN( NREG, NVOLC+9 )
            WRITE(IUNOUT,'(10X,10( A5 ,  I6)/)')
     >                  (' VOL ',IR,IR=NVOLC,NVOLM, 1)
            WRITE(IUNOUT,'(10H WEIGHT   ,10F11.5)')
     >                   (WEIG(IG,IR),IR=NVOLC,NVOLM, 1)
            NVOLC = NVOLC + 10
 310     CONTINUE
 350     CONTINUE
      ENDIF
C
      RETURN
 8002 FORMAT(//' S U R F / S U R F    C O N T R I B U T I O N S'//
     >9X,'BEGIN S',6X,'END S ',11X,'CIJ.    ',11X, 'WSPACE  ',
     >                         11X,'PROBS.  ')
 8003 FORMAT(6X,I10,5X,I10,5X,1P,E15.7,5X,E15.7,5X,E15.7 )
 9000 FORMAT(//' F U N C T I O N     V A L U E S'//
     >9X,'VOL/SUR',6X,'VALUE')
 9001 FORMAT(6X,I10,5X,F10.4)
 9004 FORMAT(//' W E I G H T E D     V A L U E S'//
     >9X,'VOL/SUR',6X,'VALUE')
 9005 FORMAT(6X,I10,5X,F10.4)
      END
